package HTML::FormHandler::I18N::en_us;
# ABSTRACT: base message file
$HTML::FormHandler::I18N::en_us::VERSION = '0.40068';
use strict;
use warnings;
use base 'HTML::FormHandler::I18N';

# Auto define lexicon
our %Lexicon = (
    '_AUTO' => 1,
  );

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

HTML::FormHandler::I18N::en_us - base message file

=head1 VERSION

version 0.40068

=head1 AUTHOR

FormHandler Contributors - see HTML::FormHandler

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2017 by Gerda Shank.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
